@@ -594,6 +594,8 @@ lo_server lo_server_new_with_proto_internal(const char *group,
                 int opt = 1;
                 setsockopt(s->sockets[0].fd, SOL_SOCKET, SO_BROADCAST,
 						   (char*)&opt, sizeof(int));
+                setsockopt(s->sockets[0].fd, IPPROTO_IP, IP_MULTICAST_LOOP,
+                           (char*)&opt, sizeof(int));
             }
         }
         if (s->sockets[0].fd == -1) {
@@ -1180,14 +1182,18 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
 
             msg_len = sc->buffer_read_offset - sc->buffer_msg_offset - sizeof(uint32_t);
 
-            // Store message length header
-            *(uint32_t*)(sc->buffer + sc->buffer_msg_offset) = htonl(msg_len);
+            // Skip empty messages, for instance for when sender is double-ENDing SLIP
+            if (msg_len)
+            {
+                // Store message length header
+                *(uint32_t*)(sc->buffer + sc->buffer_msg_offset) = htonl(msg_len);
 
-            // Advance to next message and zero the message length header
-            sc->buffer_msg_offset += msg_len + sizeof(uint32_t);
-            sc->buffer_read_offset += sizeof(uint32_t);
-            buffer_after += sizeof(uint32_t);
-            *(uint32_t*)(sc->buffer + sc->buffer_msg_offset) = 0;
+                // Advance to next message and zero the message length header
+                sc->buffer_msg_offset += msg_len + sizeof(uint32_t);
+                sc->buffer_read_offset += sizeof(uint32_t);
+                buffer_after += sizeof(uint32_t);
+                *(uint32_t*)(sc->buffer + sc->buffer_msg_offset) = 0;
+            }
 
             // Update how much memory still needs decoding.
             bytes_recv -= bytes_read;
@@ -1199,6 +1205,7 @@ int lo_server_recv_raw_stream_socket(lo_server s, int isock,
             {
                 sc->buffer_size *= 2;
                 sc->buffer = (char*)  realloc(sc->buffer, sc->buffer_size);
+                buffer_after = sc->buffer + sc->buffer_read_offset;
             }
         }
 
